@@ -23,6 +23,7 @@
 
 #include "config.h"
 
+#include <signal.h>
 #include <stdbool.h>
 #include <stdint.h>
 #include <stdio.h>
@@ -44,13 +45,13 @@
 
 #include <wayland-client.h>
 
-#include "shared/config-parser.h"
+#include <libweston/config-parser.h>
 #include "shared/helpers.h"
 #include "shared/xalloc.h"
 #include "window.h"
 
-static int option_fullscreen;
-static int option_maximize;
+static bool option_fullscreen;
+static bool option_maximize;
 static char *option_font;
 static int option_font_size;
 static char *option_term;
@@ -1123,9 +1124,9 @@ redraw_handler(struct widget *widget, void *data)
 				cairo_stroke(cr);
 			}
 
-                        /* skip space glyph (RLE) we use as a placeholder of
-                           the right half of a double-width character,
-                           because RLE is not available in every font. */
+			/* skip space glyph (RLE) we use as a placeholder of
+			   the right half of a double-width character,
+			   because RLE is not available in every font. */
 			if (p_row[col].ch == 0x200B)
 				continue;
 
@@ -2207,10 +2208,29 @@ data_source_cancelled(void *data, struct wl_data_source *source)
 	wl_data_source_destroy(source);
 }
 
+static void
+data_source_dnd_drop_performed(void *data, struct wl_data_source *source)
+{
+}
+
+static void
+data_source_dnd_finished(void *data, struct wl_data_source *source)
+{
+}
+
+static void
+data_source_action(void *data,
+		   struct wl_data_source *source, uint32_t dnd_action)
+{
+}
+
 static const struct wl_data_source_listener data_source_listener = {
 	data_source_target,
 	data_source_send,
-	data_source_cancelled
+	data_source_cancelled,
+	data_source_dnd_drop_performed,
+	data_source_dnd_finished,
+	data_source_action
 };
 
 static const char text_mime_type[] = "text/plain;charset=utf-8";
@@ -2929,6 +2949,8 @@ terminal_create(struct display *display)
 	terminal->widget = window_frame_create(terminal->window, terminal);
 	terminal->title = xstrdup("Wayland Terminal");
 	window_set_title(terminal->window, terminal->title);
+	window_set_appid(terminal->window,
+			 "org.freedesktop.weston.wayland-terminal");
 	widget_set_transparent(terminal->widget, 0);
 
 	init_state_machine(&terminal->state_machine);
@@ -3001,13 +3023,22 @@ static void
 terminal_destroy(struct terminal *terminal)
 {
 	display_unwatch_fd(terminal->display, terminal->master);
-	window_destroy(terminal->window);
 	close(terminal->master);
+
+	cairo_scaled_font_destroy(terminal->font_bold);
+	cairo_scaled_font_destroy(terminal->font_normal);
+
+	widget_destroy(terminal->widget);
+	window_destroy(terminal->window);
+
 	wl_list_remove(&terminal->link);
 
 	if (wl_list_empty(&terminal_list))
 		display_exit(terminal->display);
 
+	free(terminal->data);
+	free(terminal->data_attr);
+	free(terminal->tab_ruler);
 	free(terminal->title);
 	free(terminal);
 }
@@ -3026,10 +3057,12 @@ io_handler(struct task *task, uint32_t events)
 	}
 
 	len = read(terminal->master, buffer, sizeof buffer);
-	if (len < 0)
+	if (len < 0) {
 		terminal_destroy(terminal);
-	else
-		terminal_data(terminal, buffer, len);
+		return;
+	}
+
+	terminal_data(terminal, buffer, len);
 }
 
 static int
@@ -3068,11 +3101,12 @@ terminal_run(struct terminal *terminal, const char *path)
 		setenv("TERM", option_term, 1);
 		setenv("COLORTERM", option_term, 1);
 		if (execl(path, path, NULL)) {
-			printf("exec failed: %m\n");
+			printf("exec failed: %s\n", strerror(errno));
 			exit(EXIT_FAILURE);
 		}
 	} else if (pid < 0) {
-		fprintf(stderr, "failed to fork and create pty (%m).\n");
+		fprintf(stderr, "failed to fork and create pty (%s).\n",
+			strerror(errno));
 		return -1;
 	}
 
@@ -3105,8 +3139,9 @@ static const struct weston_option terminal_options[] = {
 int main(int argc, char *argv[])
 {
 	struct display *d;
-	struct terminal *terminal;
+	struct terminal *terminal, *tmp;
 	const char *config_file;
+	struct sigaction sigpipe;
 	struct weston_config *config;
 	struct weston_config_section *s;
 
@@ -3116,12 +3151,12 @@ int main(int argc, char *argv[])
 
 	option_shell = getenv("SHELL");
 	if (!option_shell)
-		option_shell = "/bin/bash";
+		option_shell = "/bin/sh";
 
 	config_file = weston_config_get_name_from_env();
 	config = weston_config_parse(config_file);
 	s = weston_config_get_section(config, "terminal", NULL, NULL);
-	weston_config_section_get_string(s, "font", &option_font, "mono");
+	weston_config_section_get_string(s, "font", &option_font, "monospace");
 	weston_config_section_get_int(s, "font-size", &option_font_size, 14);
 	weston_config_section_get_string(s, "term", &option_term, "xterm");
 	weston_config_destroy(config);
@@ -3137,9 +3172,18 @@ int main(int argc, char *argv[])
 		return 1;
 	}
 
+	/* Disable SIGPIPE so that paste operations do not crash the program
+	 * when the file descriptor provided to receive data is a pipe or
+	 * socket whose reading end has been closed */
+	sigpipe.sa_handler = SIG_IGN;
+	sigemptyset(&sigpipe.sa_mask);
+	sigpipe.sa_flags = 0;
+	sigaction(SIGPIPE, &sigpipe, NULL);
+
 	d = display_create(&argc, argv);
 	if (d == NULL) {
-		fprintf(stderr, "failed to create display: %m\n");
+		fprintf(stderr, "failed to create display: %s\n",
+			strerror(errno));
 		return -1;
 	}
 
@@ -3150,5 +3194,9 @@ int main(int argc, char *argv[])
 
 	display_run(d);
 
+	wl_list_for_each_safe(terminal, tmp, &terminal_list, link)
+		terminal_destroy(terminal);
+	display_destroy(d);
+
 	return 0;
 }
