@@ -50,7 +50,7 @@
  *
  * 4/ According properties, set transformation by using weston_matrix and
  *    weston_view per ivi_surfaces and ivi_layers in while loop.
- * 5/ Set damage and trigger transform by using weston_view_geometry_dirty.
+ * 5/ Set damage and trigger transform by using weston_view_add_transform
  * 6/ Schedule repaint for each view by using weston_view_schedule_repaint.
  * 7/ Notify update of properties.
  *
@@ -149,7 +149,7 @@ ivi_view_is_rendered(struct ivi_layout_view *view)
 static void
 ivi_view_destroy(struct ivi_layout_view *ivi_view)
 {
-	wl_list_remove(&ivi_view->transform.link);
+	weston_view_remove_transform(ivi_view->view, &ivi_view->transform);
 	wl_list_remove(&ivi_view->link);
 	wl_list_remove(&ivi_view->surf_link);
 	wl_list_remove(&ivi_view->pending_link);
@@ -370,7 +370,7 @@ update_opacity(struct ivi_layout_layer *ivilayer,
 	double layer_alpha = wl_fixed_to_double(ivilayer->prop.opacity);
 	double surf_alpha  = wl_fixed_to_double(ivisurf->prop.opacity);
 
-	view->alpha = layer_alpha * surf_alpha;
+	weston_view_set_alpha(view, layer_alpha * surf_alpha);
 }
 
 static void
@@ -546,13 +546,13 @@ calc_surface_to_global_matrix_and_mask_to_weston_surface(
 						     lp->dest_y,
 						     lp->dest_width,
 						     lp->dest_height };
-	struct ivi_rectangle screen_dest_rect =    { output->x,
-						     output->y,
+	struct ivi_rectangle screen_dest_rect =    { output->pos.c.x,
+						     output->pos.c.y,
 						     output->width,
 						     output->height };
 	struct ivi_rectangle layer_dest_rect_in_global =
-						   { lp->dest_x + output->x,
-						     lp->dest_y + output->y,
+						   { lp->dest_x + output->pos.c.x,
+						     lp->dest_y + output->pos.c.y,
 						     lp->dest_width,
 						     lp->dest_height };
 	struct ivi_rectangle layer_dest_rect_in_global_intersected;
@@ -569,7 +569,7 @@ calc_surface_to_global_matrix_and_mask_to_weston_surface(
 	calc_transformation_matrix(&surface_source_rect, &surface_dest_rect, m);
 	calc_transformation_matrix(&layer_source_rect, &layer_dest_rect, m);
 
-	weston_matrix_translate(m, output->x, output->y, 0.0f);
+	weston_matrix_translate(m, output->pos.c.x, output->pos.c.y, 0.0f);
 
 	/*
 	 * destination rectangle of layer in multi screens coordinate
@@ -611,24 +611,19 @@ update_prop(struct ivi_layout_view *ivi_view)
 	}
 
 	if (can_calc) {
-		wl_list_remove(&ivi_view->transform.link);
 		weston_matrix_init(&ivi_view->transform.matrix);
 
 		calc_surface_to_global_matrix_and_mask_to_weston_surface(
 			iviscrn, ivilayer, ivisurf, &ivi_view->transform.matrix, &r);
 
 		weston_view_set_mask(ivi_view->view, r.x, r.y, r.width, r.height);
-		wl_list_insert(&ivi_view->view->geometry.transformation_list,
-			       &ivi_view->transform.link);
-
+		weston_view_add_transform(ivi_view->view,
+					  &ivi_view->view->geometry.transformation_list,
+					  &ivi_view->transform);
 		weston_view_set_transform_parent(ivi_view->view, NULL);
-		weston_view_geometry_dirty(ivi_view->view);
-		weston_view_update_transform(ivi_view->view);
 	}
 
 	ivisurf->update_count++;
-
-	weston_view_schedule_repaint(ivi_view->view);
 }
 
 static bool
@@ -850,26 +845,22 @@ build_view_list(struct ivi_layout *layout)
 	 */
 	wl_list_for_each(ivi_view, &layout->view_list, link) {
 		if (!ivi_view_is_mapped(ivi_view))
-			weston_view_unmap(ivi_view->view);
+			weston_view_move_to_layer(ivi_view->view, NULL);
 	}
 
-	/* Clear view list of layout ivi_layer */
-	wl_list_init(&layout->layout_layer.view_list.link);
-
 	wl_list_for_each(iviscrn, &layout->screen_list, link) {
 		wl_list_for_each(ivilayer, &iviscrn->order.layer_list, order.link) {
-			if (ivilayer->prop.visibility == false)
-				continue;
-
 			wl_list_for_each(ivi_view, &ivilayer->order.view_list, order_link) {
-				if (ivi_view->ivisurf->prop.visibility == false)
+				if (ivilayer->prop.visibility == false ||
+				    ivi_view->ivisurf->prop.visibility == false) {
+					weston_view_move_to_layer(ivi_view->view,
+								  NULL);
 					continue;
-
-				weston_layer_entry_insert(&layout->layout_layer.view_list,
-							  &ivi_view->view->layer_link);
+				}
 
 				weston_surface_map(ivi_view->ivisurf->surface);
-				ivi_view->view->is_mapped = true;
+				weston_view_move_to_layer(ivi_view->view,
+							  &layout->layout_layer.view_list);
 			}
 		}
 	}
