@@ -220,19 +220,12 @@ ivi_shell_surface_get_label(struct weston_surface *surface,
 static void
 layout_surface_cleanup(struct ivi_shell_surface *ivisurf)
 {
-	struct weston_seat *seat;
 	assert(ivisurf->layout_surface != NULL);
 
 	/* destroy weston_surface destroy signal. */
 	if (!ivisurf->layout_surface->weston_desktop_surface)
 		wl_list_remove(&ivisurf->surface_destroy_listener.link);
 
-	wl_list_for_each(seat, &ivisurf->surface->compositor->seat_list, link) {
-		struct ivi_shell_seat *shseat = get_ivi_shell_seat(seat);
-		if (shseat->focused_ivisurf == ivisurf->layout_surface)
-			shseat->focused_ivisurf = NULL;
-	}
-
 	ivi_layout_surface_destroy(ivisurf->layout_surface);
 	ivisurf->layout_surface = NULL;
 
@@ -695,7 +688,7 @@ desktop_surface_removed(struct weston_desktop_surface *surface,
 
 static void
 desktop_surface_committed(struct weston_desktop_surface *surface,
-			  int32_t sx, int32_t sy, void *user_data)
+			  struct weston_coord_surface buf_offset, void *user_data)
 {
 	struct ivi_shell_surface *ivisurf = (struct ivi_shell_surface *)
 			weston_desktop_surface_get_user_data(surface);
@@ -778,7 +771,7 @@ desktop_surface_minimized_requested(struct weston_desktop_surface *surface,
 
 static void
 desktop_surface_set_xwayland_position(struct weston_desktop_surface *surface,
-				      int32_t x, int32_t y, void *user_data)
+				      struct weston_coord_global pos, void *user_data)
 {
 	/* Not supported */
 }
@@ -982,8 +975,10 @@ input_panel_surface_set_toplevel(struct wl_client *client,
 
 	head = weston_head_from_resource(output_resource);
 
-	ipsurf->type = INPUT_PANEL_TOPLEVEL;
-	ipsurf->output = head->output;
+	if (head) {
+		ipsurf->type = INPUT_PANEL_TOPLEVEL;
+		ipsurf->output = head->output;
+	}
 }
 
 static void
