@@ -704,8 +704,6 @@ drm_output_set_cursor(struct drm_output_state *output_state)
 		return;
 
 	if (!state->fb) {
-		pixman_region32_fini(&plane->base.damage);
-		pixman_region32_init(&plane->base.damage);
 		drmModeSetCursor(device->drm.fd, crtc->crtc_id, 0, 0, 0);
 		return;
 	}
@@ -723,9 +721,6 @@ drm_output_set_cursor(struct drm_output_state *output_state)
 		}
 	}
 
-	pixman_region32_fini(&plane->base.damage);
-	pixman_region32_init(&plane->base.damage);
-
 	if (drmModeMoveCursor(device->drm.fd, crtc->crtc_id,
 	                      state->dest_x, state->dest_y)) {
 		weston_log("failed to move cursor: %s\n", strerror(errno));
@@ -1792,7 +1787,6 @@ int
 init_kms_caps(struct drm_device *device)
 {
 	struct drm_backend *b = device->backend;
-	struct weston_compositor *compositor = b->compositor;
 	uint64_t cap;
 	int ret;
 
@@ -1804,10 +1798,7 @@ init_kms_caps(struct drm_device *device)
 		return -1;
 	}
 
-	if (weston_compositor_set_presentation_clock(compositor, CLOCK_MONOTONIC) < 0) {
-		weston_log("Error: failed to set presentation clock to CLOCK_MONOTONIC.\n");
-		return -1;
-	}
+	b->base.supported_presentation_clocks = 1 << CLOCK_MONOTONIC;
 
 	ret = drmGetCap(device->drm.fd, DRM_CAP_CURSOR_WIDTH, &cap);
 	if (ret == 0)
