@@ -167,9 +167,6 @@ headless_output_repaint(struct weston_output *output_base,
 	ec->renderer->repaint_output(&output->base, damage,
 				     output->renderbuffer);
 
-	pixman_region32_subtract(&ec->primary_plane.damage,
-				 &ec->primary_plane.damage, damage);
-
 	wl_event_source_timer_update(output->finish_frame_timer, 16);
 
 	return 0;
@@ -183,6 +180,8 @@ headless_output_disable_gl(struct headless_output *output)
 
 	weston_gl_borders_fini(&output->gl.borders, &output->base);
 
+	weston_renderbuffer_unref(output->renderbuffer);
+	output->renderbuffer = NULL;
 	renderer->gl->output_destroy(&output->base);
 
 	if (output->frame) {
@@ -252,10 +251,7 @@ headless_output_enable_gl(struct headless_output *output)
 	struct headless_backend *b = output->backend;
 	const struct weston_renderer *renderer = b->compositor->renderer;
 	const struct weston_mode *mode = output->base.current_mode;
-	struct gl_renderer_pbuffer_options options = {
-		.formats = b->formats,
-		.formats_count = b->formats_count,
-	};
+	struct gl_renderer_fbo_options options = { 0 };
 
 	if (b->decorate) {
 		/*
@@ -283,7 +279,7 @@ headless_output_enable_gl(struct headless_output *output)
 		options.fb_size.height = mode->height;
 	}
 
-	if (renderer->gl->output_pbuffer_create(&output->base, &options) < 0) {
+	if (renderer->gl->output_fbo_create(&output->base, &options) < 0) {
 		weston_log("failed to create gl renderer output state\n");
 		if (output->frame) {
 			frame_destroy(output->frame);
@@ -292,7 +288,19 @@ headless_output_enable_gl(struct headless_output *output)
 		return -1;
 	}
 
+	output->renderbuffer =
+		renderer->gl->create_fbo(&output->base, b->formats[0],
+					 options.fb_size.width,
+					 options.fb_size.height, NULL);
+	if (!output->renderbuffer)
+		goto err_renderbuffer;
+
 	return 0;
+
+err_renderbuffer:
+	renderer->gl->output_destroy(&output->base);
+
+	return -1;
 }
 
 static int
@@ -495,7 +503,7 @@ headless_destroy(struct weston_backend *backend)
 	struct weston_compositor *ec = b->compositor;
 	struct weston_head *base, *next;
 
-	weston_compositor_shutdown(ec);
+	wl_list_remove(&b->base.link);
 
 	wl_list_for_each_safe(base, next, &ec->head_list, compositor_link) {
 		if (to_headless_head(base))
@@ -508,7 +516,10 @@ headless_destroy(struct weston_backend *backend)
 	free(b->formats);
 	free(b);
 
-	cleanup_after_cairo();
+	/* XXX: cleaning up after cairo/fontconfig here might seem suitable,
+	 * but fontconfig will create additional threads which we can't wait
+	 * for -- in order to realiably de-allocate all resources, as to get a
+	 * report without any mem leaks. */
 }
 
 static const struct weston_windowed_output_api api = {
@@ -528,10 +539,10 @@ headless_backend_create(struct weston_compositor *compositor,
 		return NULL;
 
 	b->compositor = compositor;
-	compositor->backend = &b->base;
+	wl_list_insert(&compositor->backend_list, &b->base.link);
 
-	if (weston_compositor_set_presentation_clock_software(compositor) < 0)
-		goto err_free;
+	b->base.supported_presentation_clocks =
+			WESTON_PRESENTATION_CLOCKS_SOFTWARE;
 
 	b->base.destroy = headless_destroy;
 	b->base.create_output = headless_output_create;
@@ -548,56 +559,57 @@ headless_backend_create(struct weston_compositor *compositor,
 	b->formats_count = ARRAY_LENGTH(headless_formats);
 	b->formats = pixel_format_get_array(headless_formats, b->formats_count);
 
-	switch (config->renderer) {
-	case WESTON_RENDERER_GL: {
-		const struct gl_renderer_display_options options = {
-			.egl_platform = EGL_PLATFORM_SURFACELESS_MESA,
-			.egl_native_display = NULL,
-			.egl_surface_type = EGL_PBUFFER_BIT,
-			.formats = b->formats,
-			.formats_count = b->formats_count,
-		};
-		ret = weston_compositor_init_renderer(compositor,
-						      WESTON_RENDERER_GL,
-						      &options.base);
-		break;
-	}
-	case WESTON_RENDERER_PIXMAN:
-		if (config->decorate) {
-			weston_log("Error: Pixman renderer does not support decorations.\n");
-			goto err_input;
+	if (!compositor->renderer) {
+		switch (config->renderer) {
+		case WESTON_RENDERER_GL: {
+			const struct gl_renderer_display_options options = {
+				.egl_platform = EGL_PLATFORM_SURFACELESS_MESA,
+				.egl_native_display = NULL,
+				.formats = b->formats,
+				.formats_count = b->formats_count,
+			};
+			ret = weston_compositor_init_renderer(compositor,
+							      WESTON_RENDERER_GL,
+							      &options.base);
+			break;
 		}
-		ret = weston_compositor_init_renderer(compositor,
-						      WESTON_RENDERER_PIXMAN,
-						      NULL);
-		break;
-	case WESTON_RENDERER_AUTO:
-	case WESTON_RENDERER_NOOP:
-		if (config->decorate) {
-			weston_log("Error: no-op renderer does not support decorations.\n");
-			goto err_input;
+		case WESTON_RENDERER_PIXMAN:
+			if (config->decorate) {
+				weston_log("Error: Pixman renderer does not support decorations.\n");
+				goto err_input;
+			}
+			ret = weston_compositor_init_renderer(compositor,
+							      WESTON_RENDERER_PIXMAN,
+							      NULL);
+			break;
+		case WESTON_RENDERER_AUTO:
+		case WESTON_RENDERER_NOOP:
+			if (config->decorate) {
+				weston_log("Error: no-op renderer does not support decorations.\n");
+				goto err_input;
+			}
+			ret = noop_renderer_init(compositor);
+			break;
+		default:
+			weston_log("Error: unsupported renderer\n");
+			break;
 		}
-		ret = noop_renderer_init(compositor);
-		break;
-	default:
-		weston_log("Error: unsupported renderer\n");
-		break;
-	}
 
-	if (ret < 0)
-		goto err_input;
-
-	if (compositor->renderer->import_dmabuf) {
-		if (linux_dmabuf_setup(compositor) < 0) {
-			weston_log("Error: dmabuf protocol setup failed.\n");
+		if (ret < 0)
 			goto err_input;
-		}
-	}
 
-	/* Support zwp_linux_explicit_synchronization_unstable_v1 to enable
-	 * testing. */
-	if (linux_explicit_synchronization_setup(compositor) < 0)
-		goto err_input;
+		if (compositor->renderer->import_dmabuf) {
+			if (linux_dmabuf_setup(compositor) < 0) {
+				weston_log("Error: dmabuf protocol setup failed.\n");
+				goto err_input;
+			}
+		}
+
+		/* Support zwp_linux_explicit_synchronization_unstable_v1 to enable
+		 * testing. */
+		if (linux_explicit_synchronization_setup(compositor) < 0)
+			goto err_input;
+	}
 
 	ret = weston_plugin_api_register(compositor, WESTON_WINDOWED_OUTPUT_API_NAME,
 					 &api, sizeof(api));
@@ -612,9 +624,8 @@ headless_backend_create(struct weston_compositor *compositor,
 err_input:
 	if (b->theme)
 		theme_destroy(b->theme);
-
-	weston_compositor_shutdown(compositor);
 err_free:
+	wl_list_remove(&b->base.link);
 	free(b);
 	return NULL;
 }
