@@ -190,11 +190,8 @@ static void
 weston_desktop_seat_popup_grab_touch_down(struct weston_touch_grab *grab,
 					  const struct timespec *time,
 					  int touch_id,
-					  wl_fixed_t sx, wl_fixed_t sy)
+					  struct weston_coord_global pos)
 {
-	struct weston_coord_global pos;
-
-	pos.c = weston_coord_from_fixed(sx, sy);
 	weston_touch_send_down(grab->touch, time, touch_id, pos);
 }
 
@@ -210,11 +207,8 @@ static void
 weston_desktop_seat_popup_grab_touch_motion(struct weston_touch_grab *grab,
 					    const struct timespec *time,
 					    int touch_id,
-					    wl_fixed_t sx, wl_fixed_t sy)
+					    struct weston_coord_global pos)
 {
-	struct weston_coord_global pos;
-
-	pos.c = weston_coord_from_fixed(sx, sy);
 	weston_touch_send_motion(grab->touch, time, touch_id, pos);
 }
 
@@ -538,9 +532,6 @@ weston_desktop_seat_popup_grab_add_surface(struct weston_desktop_seat *seat,
 
 	wl_list_insert(&seat->popup_grab.surfaces, link);
 
-	if (!seat->popup_grab.keyboard.keyboard)
-		return;
-
 	desktop_surface =
 		weston_desktop_seat_popup_grab_get_topmost_surface(seat);
 	surface = weston_desktop_surface_get_surface(desktop_surface);
@@ -561,9 +552,6 @@ weston_desktop_seat_popup_grab_remove_surface(struct weston_desktop_seat *seat,
 		struct weston_desktop_surface *desktop_surface;
 		struct weston_surface *surface;
 
-		if (!seat->popup_grab.keyboard.keyboard)
-			return;
-
 		desktop_surface =
 			weston_desktop_seat_popup_grab_get_topmost_surface(seat);
 		surface = weston_desktop_surface_get_surface(desktop_surface);
