@@ -25,7 +25,6 @@
 
 #include "config.h"
 
-#include <assert.h>
 #include <errno.h>
 #include <stdint.h>
 #include <unistd.h>
@@ -35,8 +34,11 @@
 
 #include <libweston/libweston.h>
 #include "linux-dmabuf.h"
+#include "linux-dmabuf-unstable-v1-server-protocol.h"
 #include "shared/os-compatibility.h"
+#include "shared/helpers.h"
 #include "libweston-internal.h"
+#include "shared/weston-assert.h"
 #include "shared/weston-drm-fourcc.h"
 
 static void
@@ -93,8 +95,8 @@ params_add(struct wl_client *client,
 		return;
 	}
 
-	assert(buffer->params_resource == params_resource);
-	assert(!buffer->buffer_resource);
+	weston_assert_ptr_eq(buffer->compositor, buffer->params_resource, params_resource);
+	weston_assert_ptr_is_null(buffer->compositor, buffer->buffer_resource);
 
 	if (plane_idx >= MAX_DMABUF_PLANES) {
 		wl_resource_post_error(params_resource,
@@ -120,8 +122,7 @@ params_add(struct wl_client *client,
 	if (wl_resource_get_version(params_resource) < ZWP_LINUX_DMABUF_V1_MODIFIER_SINCE_VERSION)
 		buffer->attributes.modifier[plane_idx] = DRM_FORMAT_MOD_INVALID;
 	else
-		buffer->attributes.modifier[plane_idx] = ((uint64_t)modifier_hi << 32) |
-							 modifier_lo;
+		buffer->attributes.modifier[plane_idx] = u64_from_u32s(modifier_hi, modifier_lo);
 
 	buffer->attributes.n_planes++;
 }
@@ -143,8 +144,8 @@ destroy_linux_dmabuf_wl_buffer(struct wl_resource *resource)
 	struct linux_dmabuf_buffer *buffer;
 
 	buffer = wl_resource_get_user_data(resource);
-	assert(buffer->buffer_resource == resource);
-	assert(!buffer->params_resource);
+	weston_assert_ptr_eq(buffer->compositor, buffer->buffer_resource, resource);
+	weston_assert_ptr_is_null(buffer->compositor, buffer->params_resource);
 
 	if (buffer->user_data_destroy_func)
 		buffer->user_data_destroy_func(buffer);
@@ -173,8 +174,8 @@ params_create_common(struct wl_client *client,
 		return;
 	}
 
-	assert(buffer->params_resource == params_resource);
-	assert(!buffer->buffer_resource);
+	weston_assert_ptr_eq(buffer->compositor, buffer->params_resource, params_resource);
+	weston_assert_ptr_is_null(buffer->compositor, buffer->buffer_resource);
 
 	/* Switch the linux_dmabuf_buffer object from params resource to
 	 * eventually wl_buffer resource.
@@ -778,16 +779,18 @@ weston_dmabuf_feedback_send(struct weston_dmabuf_feedback *dmabuf_feedback,
  * is dynamic and can change throughout compositor's life. These changes results
  * in the need to resend the feedback events to clients.
  *
+ * @param compositor The weston compositor
  * @param dmabuf_feedback The weston_dmabuf_feedback object
  * @param format_table The dma-buf feedback formats table
  */
 WL_EXPORT void
-weston_dmabuf_feedback_send_all(struct weston_dmabuf_feedback *dmabuf_feedback,
+weston_dmabuf_feedback_send_all(struct weston_compositor *compositor,
+				struct weston_dmabuf_feedback *dmabuf_feedback,
 				struct weston_dmabuf_feedback_format_table *format_table)
 {
 	struct wl_resource *res;
 
-	assert(!wl_list_empty(&dmabuf_feedback->resource_list));
+	weston_assert_true(compositor, !wl_list_empty(&dmabuf_feedback->resource_list));
 	wl_resource_for_each(res, &dmabuf_feedback->resource_list)
 		weston_dmabuf_feedback_send(dmabuf_feedback,
 					    format_table, res, false);
@@ -940,11 +943,13 @@ err:
  * protocol interface, returns the linux_dmabuf_buffer object. This can
  * be used as a type check for a wl_buffer.
  *
+ * \param compositor The weston compositor.
  * \param resource A wl_buffer resource.
  * \return The linux_dmabuf_buffer if it exists, or NULL otherwise.
  */
 WL_EXPORT struct linux_dmabuf_buffer *
-linux_dmabuf_buffer_get(struct wl_resource *resource)
+linux_dmabuf_buffer_get(struct weston_compositor *compositor,
+			struct wl_resource *resource)
 {
 	struct linux_dmabuf_buffer *buffer;
 
@@ -956,9 +961,9 @@ linux_dmabuf_buffer_get(struct wl_resource *resource)
 		return NULL;
 
 	buffer = wl_resource_get_user_data(resource);
-	assert(buffer);
-	assert(!buffer->params_resource);
-	assert(buffer->buffer_resource == resource);
+	weston_assert_ptr(compositor, buffer);
+	weston_assert_ptr_is_null(compositor, buffer->params_resource);
+	weston_assert_ptr_eq(compositor, buffer->buffer_resource, resource);
 
 	return buffer;
 }
@@ -985,7 +990,7 @@ linux_dmabuf_buffer_set_user_data(struct linux_dmabuf_buffer *buffer,
 				  void *data,
 				  dmabuf_user_data_destroy_func func)
 {
-	assert(data == NULL || buffer->user_data == NULL);
+	weston_assert_true(buffer->compositor, data == NULL || buffer->user_data == NULL);
 
 	buffer->user_data = data;
 	buffer->user_data_destroy_func = func;
@@ -1043,7 +1048,7 @@ bind_linux_dmabuf(struct wl_client *client,
 
 	/* If we got here, it means that the renderer is able to import dma-buf
 	 * buffers, and so it must have get_supported_formats() set. */
-	assert(compositor->renderer->get_supported_formats != NULL);
+	weston_assert_ptr(compositor, compositor->renderer->get_supported_formats);
 	supported_formats = compositor->renderer->get_supported_formats(compositor);
 
 	wl_array_for_each(fmt, &supported_formats->arr) {
@@ -1118,12 +1123,12 @@ linux_dmabuf_buffer_send_server_error(struct linux_dmabuf_buffer *buffer,
 	struct wl_resource *display_resource;
 	uint32_t id;
 
-	assert(buffer->buffer_resource);
+	weston_assert_ptr(buffer->compositor, buffer->buffer_resource);
 	id = wl_resource_get_id(buffer->buffer_resource);
 	client = wl_resource_get_client(buffer->buffer_resource);
 	display_resource = wl_client_get_object(client, 1);
 
-	assert(display_resource);
+	weston_assert_ptr(buffer->compositor, display_resource);
 	wl_resource_post_error(display_resource,
 			       WL_DISPLAY_ERROR_INVALID_OBJECT,
 			       "linux_dmabuf server error with "
