@@ -40,10 +40,12 @@ fixture_setup(struct weston_test_harness *harness)
 
 	compositor_setup_defaults(&setup);
 
+	setup.shell = SHELL_TEST_DESKTOP;
+
 	/* We need to use the pixman renderer, since a few of the tests depend
 	 * on the renderer holding onto a surface buffer until the next one
 	 * is committed, which the noop renderer doesn't do. */
-	setup.renderer = RENDERER_PIXMAN;
+	setup.renderer = WESTON_RENDERER_PIXMAN;
 
 	return weston_test_harness_execute_as_client(harness, &setup);
 }
